package com.xy.job;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xy.collections.map.JHashMap;
import com.xy.collections.map.JMap;
import com.xy.device.EnumDeviceCharging;
import com.xy.dto.MercMsgInfoDto;
import com.xy.entity.SysDictRedis;
import com.xy.mapper.DeviceSimMapper;
import com.xy.service.MercMsgInfoService;
import com.xy.sys.EnumMercCostMsgConfig;
import com.xy.utils.*;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * 设备流量卡job
 */
@Slf4j
@Component
@AllArgsConstructor
public class DeviceSimJob {

    private DeviceSimMapper deviceSimMapper;

    private MercMsgInfoService mercMsgInfoService;

    /**
     * 设备流量卡过期消息
     *
     * @return
     */
    @XxlJob("deviceSimTimeout")
    public ReturnT<String> deviceSimTimeout() {
        String now = DataTime.getSring();
        Map<String, SysDictRedis> stringSysDictRedisMap = SysDictUtils.get(EnumMercCostMsgConfig.Code.CODE.getCode());
        String tel = stringSysDictRedisMap.get(EnumMercCostMsgConfig.CUSTOM_TEL.getCode()).getValue();
        String mmiTime = DataTime.getStringAround(0, -1, 0, 0, 0, 0, now);
        //数据逻辑
        FunctionUtils.ParamsResult<Tuple.Tuple4<Integer, JMap<String, Object>, JMap<String, Object>, Integer>, Boolean> datas = tuple -> {
            Integer current = tuple.getV1();
            JMap<String, Object> params = tuple.getV2();
            JMap<String, Object> msg = tuple.getV3();
            Integer workType = tuple.getV4();
            //查询数据
            IPage<Long> longIPage = deviceSimMapper.deviceSimTimeout(Page.of(current, 100), params);
            List<Long> mercIds = longIPage.getRecords();
            if (!Emptys.check(mercIds)) {
                return false;
            }
            //发送消息
            MsgUtils.sendMsg(17L, mercIds)
                    .official(userInfo -> new MsgUtils.MsgInfo(userInfo.getMpOpenid(), msg))
                    .mxr(userInfo -> new MsgUtils.MsgInfo(String.valueOf(userInfo.getAuthorizeUserId()), msg))
                    .builder();
            //写入记录
            mercMsgInfoService.save(new MercMsgInfoDto.Save()
                    .setMercId(mercIds)
                    .setType(2)
                    .setWorkType(workType)
            );
            return true;
        };
        //即将过期
        FunctionUtils.NoParamsNoResult timeout1 = () -> {
            int day = SysDictUtils.getValue(EnumDeviceCharging.Code.CODE.getCode(), EnumDeviceCharging.N_200.getCode(), Integer.class);
            String end = DataTime.getStringAround(0, 0, day, 0, 0, 0, now);
            JMap<String, Object> map = new JHashMap<String, Object>()
                    .set("mmiTime", mmiTime)
                    .set("workType", 1)
                    .set("begin", now)
                    .set("end", end);
            JMap<String, Object> sendParams = new JHashMap<String, Object>()
                    .set("content", String.format("尊敬的客户，您有%s即将到期，为避免设备停机，请及时续费，客服电话%s", "设备流量卡", tel));
            RecursionUtils.recursion(current -> datas.run(new Tuple.Tuple4<>(current, map, sendParams, 1)));
        };
        //欠费
        FunctionUtils.NoParamsNoResult timeout2 = () -> {
            int day = Integer.parseInt(stringSysDictRedisMap.get(EnumMercCostMsgConfig.ARREARAGE_DAY.getCode()).getValue());
            String time1 = DataTime.getStringAround(0, 0, day, 0, 0, 0, now);
            JMap<String, Object> map = new JHashMap<String, Object>()
                    .set("mmiTime", mmiTime)
                    .set("workType", 2)
                    .set("end", now);
            JMap<String, Object> sendParams = new JHashMap<String, Object>()
                    .set("content", String.format("尊敬的客户，截止%s您的%s已欠费，如您在%s前未完成续费，您的设备将停止交易，客服电话%s", now, "设备流量卡", time1, tel));
            RecursionUtils.recursion(current -> datas.run(new Tuple.Tuple4<>(current, map, sendParams, 2)));
        };
        timeout1.run();
        timeout2.run();
        return ReturnT.SUCCESS;
    }
}
